	module functions

!
!	Auxiliary functions needed for calculating with C1 and C2 tension splines
!
	
	implicit none

	contains



	double precision function f (x)

!
!	The function f(x)=(sinh(x)-x)/(2*x*(sinh(x/2))**2)
!

	implicit none

	double precision, intent(in) :: x
	double precision :: temp,num,den
	double precision, parameter :: a(0:3)=(/ 0.3333333333333333d0, 1997/237420.d0, &
											1343/1.440348d7, 21409/1.711133424d11 /)
	double precision, parameter :: b(0:3)=(/ 1.d0, 309/5276.d0, 2143/2.05764d6, &
											57623/1.140755616d10 /)
	integer :: n,i

	if (x>=0.0d0 .and. x<0.25d0) then

!	Pade approximation

		n=3
		temp=0.0d0
		do i=n,0,-1
			temp=a(i)+x*x*temp
		enddo
		num=temp

		n=3
		temp=0.0d0
		do i=n,0,-1
			temp=b(i)+x*x*temp
		enddo
		den=temp

		f=num/den

	elseif (x<50.0d0) then
		
		temp=sinh(x/2)
		f=(sinh(x)-x)/(2*x*temp*temp)

	else

		f=1.0d0/x

	endif


	end function f



	double precision function g (x)

!
!	The function g(x)=exp(x)*(x*cosh(x)-sinh(x))/(x*(sinh(x))**2)
!

	implicit none

	double precision, intent(in) :: x
	double precision :: temp,num,den
	double precision, parameter :: a(0:3)=(/ 0.3333333333333333d0, 2500687033/6.316180974d10, &
											-6652417649/3.83181645756d12, & 
											96540906137/3.5016907319856d15 /)
	double precision, parameter :: b(0:3)=(/ 1.d0, 494218149/1.403595772d9, &
											21921061991/5.4740235108d11, & 
											4466870855383/3.03479863438752d15 /)
	integer :: n,i


	if (x>=0.0d0 .and. x<0.25d0) then

!	Pade approximation

		n=3
		temp=0.0d0
		do i=n,0,-1
			temp=a(i)+x*x*temp
		enddo
		num=temp

		n=3
		temp=0.0d0
		do i=n,0,-1
			temp=b(i)+x*x*temp
		enddo
		den=temp

		g=exp(x)*num/den

	elseif (x<19.0d0) then
		
		temp=sinh(x)
		g=2*(x*cosh(x)-temp)/(x*temp*(1-exp(-2*x)))

	elseif (x<40.0d0) then
		
		temp=exp(-2*x)
		g=2*(x*(1.0d0+temp)+temp-1.0d0)/(x*(1.0d0+exp(-4*x)-2*temp))

	else

		g=2*(x-1.0d0)/x

	endif


	end function g



	double precision function u (h,k,p)

!
!	The function u(x,y)=exp(y)*sinh(x)/sinh(x+y)
!

	implicit none

	double precision, intent(in) :: h,k,p
	double precision, parameter :: pp=19.0d0,e=1.0d-16
	double precision :: x,y

	if (p==0.0d0) then
		if (h/=0.0d0 .or. k/=0.0d0) then
			u=h/(h+k)
		else
			u=0.0d0
		endif
		return
	endif

	x=p*h
	y=p*k

	if (x==0.0d0) then

		u=0.0d0
		return

	elseif (y==0.0d0) then

		u=1.0d0
		return

	elseif (x<pp .and. y<pp-x) then

		if (x>1.0d0) then
			u=(1.0d0-exp(-2*x))/(1.0d0-exp(-2*(x+y)))
		else
			u=exp(y)*sinh(x)/sinh(x+y)
		endif

	else

		u=(1-exp(-2*x))

	endif


	end function u

!
!	Next 3 functions are needed for calculating the integrals (or relevant part of them) 
!	of B-splines associated to the 1. reduced system 
!

	double precision function CC3 (h1,h2,h3,p1,p2,p3)

!
!	The function CC3 is needed for calculating the integrals of B-splines associated to 
!	the 1. reduced system 
!

	implicit none

	double precision, intent(in) :: h1,h2,h3,p1,p2,p3
	double precision :: half,pp1,pp2

	half=h2/2
	pp1=max(p1,p2)
	pp2=max(p2,p3)

	CC3=u(h1/2,half,pp1)*(h1*f(p1*h1)+h2*f(p2*half*2))+h2*g(p2*half)+u(h3/2,half,pp2)* &
		(h2*f(p2*half*2)+h3*f(p3*h3))


	end function CC3



	double precision function C31 (h1,h3,p1,p3)

	implicit none

	double precision, intent(in) :: h1,h3,p1,p3

	C31=h1*f(p1*h1)+h3*f(p3*h3)

	end function C31



	double precision function C32 (h,p)

	implicit none

	double precision, intent(in) :: h,p
	double precision :: ph

	ph=p*h
	C32=h*(1.0d0+exp(-ph))*g(ph/2)/2

	end function C32
 


	double precision function divth1 (h,k,p)

!
!	The function divth1(h,k,p)=tanh(p*h)/tanh(p*k)
!

	implicit none

	double precision, intent (in) :: h,k,p

	if (p==0.0d0) then
		divth1=h/k
	else
		divth1=tanh(p*h)/tanh(p*k)
	endif

	end function divth1



	double precision function divth2 (h,k,l,p)

!
!	The function divth2(h,k,l,p)=tanh(p*h)/(tanh(p*h)+tanh(p*k))
!	l=k+p
!

	implicit none

	double precision, intent (in) :: h,k,l,p
	double precision :: temp

	if (p==0.0d0) then
		divth2=h/l
	else
		temp=tanh(p*h)
		divth2=temp/(temp+tanh(p*k))
	endif
		
	end function divth2



	double precision function s (x,y)

!
!	The function s(x,y)=x*tanh(y)/(y*tanh(x))
!

	implicit none

	double precision, intent(in) :: x,y
	double precision, parameter :: p=19.0d0

	if (x<1.0d-9 .and. y<1.0d-9) then
		s=1.0d0
	elseif (x==0) then
		s=tanh(y)/y
	elseif (y==0) then
		s=x/tanh(x)
	elseif (x<p .and. y<p) then
		s=x*tanh(y)/(y*tanh(x))
	elseif (y>p) then
		s=x/(y*tanh(x))
	elseif (x>p) then
		s=x*tanh(y)/y
	endif

	end function s



	double precision function KC2div (h,k,p,q)

!
!	The function w(h,k,p,q)=1/(1+k*s(p*h/2,q*k/2)/h)
!

	implicit none 

	double precision, intent(in) :: h,k,p,q

	if (h==0.0d0) then
		KC2div=0.0d0
	else
		KC2div=1.0d0/(1.0d0+k*s(p*h/2,q*k/2)/h)
	endif

	end function KC2div



	double precision function KC2 (x)

!
!	The function KC2(x)=tanh(x)/x, needed for calculating the integral of the 2. order 
!	(C2) tension B-spline
!

	implicit none

	double precision, intent(in) :: x

	if (x<1.0d-9) then
		KC2=1.0d0
	else
		KC2=tanh(x)/x
	endif

	end function KC2



	double precision function KC3 (h,p)

!
!	The integral of the 3. order (C2) tension B-spline
!

	implicit none

	double precision, intent(in) :: h(3),p(3)
	double precision :: nula=0.0d0

	KC3=KC2div(h(1),h(2),p(1),p(2))*C31(h(1),h(2),p(1),p(2))+ &
	    C32(h(2),p(2))+KC2div(h(3),h(2),p(3),p(2))* &
		C31(h(2),h(3),p(2),p(3))

	end function KC3



	double precision function div0 (x,y)

!
!	Divison of x by y, with the 0/0 problem treated by using de Boor's maxime
!

	implicit none

	double precision, intent(in) :: x,y

	if (x==0) then
		div0=0.0d0
	else
		div0=x/y
	endif

	end function div0



	subroutine gamma4 (h,p,d)

!
!	The coefficients of the quasy-Oslo type algorithm which connects
!	C2 tension B-splines with C1 tension B-splines
!

	implicit none

	double precision, intent(in) :: h(4),p(4)
	double precision, intent(out) :: d(4)
	double precision :: temp1,temp2,temp3,naz,c

!	Calculating the first two coefficients

	c=C31(h(2),h(3),p(2),p(3))
	temp1=KC2div(h(1),h(2),p(1),p(2))*C31(h(1),h(2),p(1),p(2))
	temp2=C32(h(2),p(2))
	temp3=KC2div(h(3),h(2),p(3),p(2))*c
	naz=temp1+temp2+temp3
	d(1)=div0(temp1,naz)
	d(2)=div0(temp1+temp2,naz)

!	Calculating the last two coefficients

	temp1=KC2div(h(2),h(3),p(2),p(3))*c
	temp2=C32(h(3),p(3))
	temp3=KC2div(h(4),h(3),p(4),p(3))*C31(h(3),h(4),p(3),p(4))
	naz=temp1+temp2+temp3
	d(3)=div0(temp2+temp3,naz)
	d(4)=div0(temp3,naz)

	end subroutine gamma4



	double precision function gamma4d1 (h,p)

!
!	Only the 1. coefficient of gamma4
!

	implicit none

	double precision, intent(in) :: h(3),p(3)
	double precision :: nula=0.0d0,temp1,temp2,temp3

	temp1=KC2div(h(1),h(2),p(1),p(2))*C31(h(1),h(2),p(1),p(2))
	temp2=C32(h(2),p(2))
	temp3=KC2div(h(3),h(2),p(3),p(2))*C31(h(2),h(3),p(2),p(3))
	gamma4d1=div0(temp1,temp1+temp2+temp3)

	end function gamma4d1



	double precision function gamma4d4 (h,p)

!
!	Only the 4. coefficient of gamma4
!

	implicit none

	double precision, intent(in) :: h(3),p(3)
	double precision :: nula=0.0d0,temp1,temp2,temp3

	temp1=KC2div(h(1),h(2),p(1),p(2))*C31(h(1),h(2),p(1),p(2))
	temp2=C32(h(2),p(2))
	temp3=KC2div(h(3),h(2),p(3),p(2))*C31(h(2),h(3),p(2),p(3))
	gamma4d4=div0(temp3,temp1+temp2+temp3)

	end function gamma4d4



	end module functions