	subroutine tension4C1 (x,xx,px,ileft,cx,dim,spl)

!
!   Geeneralized de Boor algorithm for calculating Chebyshev tension
!   spline curve
!
!   INPUT:
!       x     = double precision variable, the point at which the
!               spline curve is evaluated
!       xx    = rank-one double precision array of size dimx, the
!               increasing array of knots, assumed to fulfill the
!               conditions:
!               xx(1)=xx(2)<xx(3)<...<xx(i)<xx(i+1)<...<xx(dimx-1)=
!               xx(dimx); xx(3),...,xx(dimx-2) is the partition of
!               [xx(1),xx(dimx)]; the associated extended
!               partition T has all interior knots of multiplicity 2,
!               and the boundary points of multiplicity 4
!       px    = rank-one double precision array of size dimx-1, the
!               array of tension parameters, assumed to be
!               nonnegative: px(i)>=0 is the tension parameter on
!               the subinterval [xx(i),xx(i+1));
!               px(1) and px(dimx-1) can be arbitrary
!       ileft = integer variable, the index of the array xx such
!               that x is contained in the subinterval
!               [xx(ileft),xx(ileft+1)]
!       cx    = rank-two double precision array of shape dim by
!               2*dimx-4, the array of de Boor points associated
!               with the extended partition T;
!               cx(1:dim,i) corresponds to the B-spline with the
!               support [T(i),T(i+4)]
!       dim   = integer variable, the extent of the first dimension
!               in the array cx, assumed to be positive
!
!   AUXILIARY TERM:
!      dimx   = size(xx)
!
!   OUTPUT:
!       spl   = rank-one double precision array of size dim, the
!               output value of the Chebyshev tension spline curve
!               at the point x
!


	use functions

	implicit none

	integer, intent(in) :: ileft,dim
	double precision, intent(in) :: x,xx(*),px(*),cx(dim,*)	
	double precision, intent(out) :: spl(dim)

	double precision, parameter :: zero=0.0d0
	double precision :: t(-1:2),p(-1:1),c(dim,-3:0)	! t(0)<=x<=t(1)
	double precision :: temp1,temp2,temp3,temp4,h(-1:1),xa,xb,u1,u2
	double precision :: C1,C2,C3,C4,C5,C6,C7,C8,C9,C10,C11,C12
	integer :: ileftc,i

!
!	Storing only relevant data
!
	t(-1:2)=xx(ileft-1:ileft+2)
	p(-1:1)=px(ileft-1:ileft+1)
	ileftc=2*ileft
	c(1:dim,-3:0)=cx(1:dim,ileftc-3:ileftc)
!
!	Calculating the length of subintervals involved
!
	do i=-1,1
		h(i)=t(i+1)-t(i)
	enddo
!
!   If x is equal to the left end point of subinterval
!   [xx(ileft),xx(ileft+1)], then the point x must be inserted only
!   once...
!
	if (x==t(0)) then

		if (t(-1)==t(0)) then
			spl=c(1:dim,-3)
			return
		endif

		temp1=CC3(zero,zero,h(0),zero,zero,p(0))
		temp2=CC3(h(-1),zero,zero,p(-1),zero,zero)
		spl=(c(1:dim,-3)*temp1+c(1:dim,-2)*temp2)/CC3(h(-1),zero,h(0),p(-1),zero,p(0))
		return
!
!	... and also if x is equal to the right end point
!
	elseif (x==t(1)) then

		if (t(1)==t(2)) then
			spl=c(1:dim,0)
			return
		endif

		temp1=CC3(zero,zero,h(1),zero,zero,p(1))
		temp2=CC3(h(0),zero,zero,p(0),zero,zero)
		spl=(c(1:dim,-1)*temp1+c(1:dim,0)*temp2)/CC3(h(0),zero,h(1),p(0),zero,p(1))
		return

	endif
!
!   If x is in the interior of [xx(ileft),xx(ileft+1)], then the
!   generalized de Boor algorithm of order 4 is performed, where the
!   point x is inserted 3 times, but first, 12 integrals of
!   B-splines of order 3 are calculated
!
	xa=x-t(0)
	xb=t(1)-x
	u1=u(xb/2,xa/2,p(0))
	u2=u(xa/2,xb/2,p(0))
	C1=CC3(zero,zero,xb,zero,zero,p(0))
	C2=CC3(h(-1),zero,h(0),p(-1),zero,p(0))
	C3=CC3(zero,xa,xb,zero,p(0),p(0))
	C4=CC3(zero,xa,zero,zero,p(0),zero)
	C5=CC3(xa,xb,zero,p(0),p(0),zero)
	C6=CC3(xa,zero,xb,p(0),zero,p(0))
	C7=CC3(zero,h(0),zero,zero,p(0),zero)
	C8=CC3(xa,zero,zero,p(0),zero,zero)
	C9=CC3(zero,xb,zero,zero,p(0),zero)
	C10=CC3(h(0),zero,h(1),p(0),zero,p(1))
	C11=CC3(h(-1),zero,xa,p(-1),zero,p(0))
	C12=CC3(xb,zero,h(1),p(0),zero,p(1))
!
!	Calculation of 4 nontrivial B-splines at the point x
!
	temp1=u1*C1/C2
	temp2=C1*C11/(C3*C2) + (C1*C4*C5/C3 + C8*C9)/(C6*C7)
	temp3=(C1*C4 + C8*C9*C3/C5)/(C6*C7) + C8*C12/(C5*C10)
	temp4=u2*C8/C10
!
!   The final value of the Chebyshev tension spline curve
!   represented as the linear combination of B-splines
!
	spl=u1*(exp(-p(0)*xa)*c(1:dim,-3)*temp1+c(1:dim,-2)*temp2)+ &
		u2*(c(1:dim,-1)*temp3+exp(-p(0)*xb)*c(1:dim,0)*temp4)


	end subroutine tension4C1
		
